/*
 * AutoDAO - Generic DAO on steroids implementation for Java.
 *
 * Copyright 2008-2010  Marat Radchenko <slonopotamus@users.sourceforge.net>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.autodao;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import java.util.Collection;
import java.util.List;

/**
 * Annotation for DAO finder methods.
 * @since AutoDAO 0.5
 */
@Documented
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface Finder {
  /**
   * Use this parameter to specify named query name.
   * @return named query name.
   */
  String queryName() default "";

  /**
   * Use this parameter to specify HQL query.
   * @return HQL query.
   */
  String query() default "";

  /**
   * Use this parameter to specify SQL native query name.
   * @return SQL query name.
   * @since AutoDAO 0.9
   */
  String sqlQueryName() default "";

  /**
   * Use this parameter to specify SQL native query.
   * @return SQL query.
   * @since AutoDAO 0.9
   */
  String sqlQuery() default "";

  /**
   * Use this parameter if finder method wants to return custom collection type.
   * <p/>
   * The only requirements for specified type are: no-arg constructor and implemented {@link
   * Collection#addAll(java.util.Collection)} method.
   * <p/>
   * Example:
   * <pre>
   * &#64;Finder(query = "from Book where id in (:ids)", returnAs = HashSet.class)
   * Set&lt;Book&gt; findByIds(@Named("ids") Set&lt;Long&gt; ids);
   * </pre>
   * @return result collection type.
   */
  Class<? extends Collection> returnAs() default List.class;
}
